;=========================================
;========== folderMenu.ahk ===============
;=========================================
;displays a folder structure as navigatable right-click-menu
;use isChromeChecker to check all open chrome tabs, if they
;contain parts of the .url in the window name it won't open twice

rootFolder := "C:\Users\Slyce\Desktop\root\"
shortCut=F1
isChromeChecker := true ;check all open chrome tabs if window is already open
firstName := "root-kun"
colBG := "white"

;=========================================
;============== show Menu ================
;=========================================
Hotkey,%shortCut%,showMen
return

showMen:
{
  if(menu_fromfiles("filelist", firstName, "SHOWPATH", rootFolder, "*", "thismenu", 1))
  {
    ;optionally: hide the checkbox left -> makes menu more compact
    HMENU:=Menu_GetMenuByName("thismenu")
    Menu_RemoveCheckMarks(HMENU, 1)

    ;show menu
    Menu, thismenu, Color, %colBG%
    Menu, thismenu, Show
  }
  else
    msgbox, No files were found.
}
return

;=========================================
;============== click Item ===============
;=========================================
SHOWPATH:
  curpath := menu_itempath("filelist", rootFolder)
  if(curpath!="\"){
    ;dict.cc checker
    if(substr(curpath, -7)=="dict.url"){
      openSuffix("dict")
      return
    }
    ;gmaps checker
    if(substr(curpath, -7)=="maps.url"){
      openSuffix("maps")
      return
    }
    ;gdrive checker
    if(substr(curpath, -14)=="driveSearch.url"){
      openSuffix("drive")
      return
    }

    ;chrome tab checker
    if(isChromeChecker && substr(curpath, -3)==".url"){
      ;chrome not even open, so just open the .url file
      IfWinNotExist ahk_exe chrome.exe
      {
        Run, %curpath%
        Return
      }
      else{
        searchTab()
      }
    }else{
      ;simply open the file
      Run, %curpath%
    }
  }
return

searchTab(){
  curpath := menu_itempath("filelist", rootFolder)

  ;chrome is open, check all tabs:
  WinActivate, ahk_exe chrome.exe
  WinGetActiveTitle, StartingTitle

  ;get "google" in the case of ...interent/google.url
  stringSplitArray := StrSplit(curpath, "\", ".url") ;omits .url ending
  searchedTab := stringSplitArray[stringSplitArray.MaxIndex()]

  ;loop through each of the browser windows
  Loop
  {
    send {Control down}{Tab}{Control Up}
      Sleep 200
      WinGetTitle, currTitle, A
      If InStr(currTitle, searchedTab)
        Return
      WinGetActiveTitle, CurrentTabTitle
      If (CurrentTabTitle == StartingTitle)
        break
  }

  ;searched tab not found, so open the .url
  Run, %curpath%
  return
}

openSuffix(type){
  if(type=="dict"){
    baseAdress := "https://pocket.dict.cc/?s="
    dummyText := "apple"
  }
  if(type=="maps"){
    baseAdress := "https://www.google.at/maps/dir/Stephansplatz/"
    dummyText := "TargetAdress"
  }
  if(type=="drive"){
    baseAdress := "https://drive.google.com/drive/search?q="
    dummyText := "pitch"
  }

  InputBox, UserInput, search window, search:,,,,,,,,%dummyText%

  if ErrorLevel
    return
  else{
    Run, %baseAdress%%UserInput%
  }
}


;=========================================
;========== menu making code =============
;=========================================
;https://autohotkey.com/board/topic/95219-dynamicfilemenuahk/
menu_fromfiles(submenuname, menutitle, whatsub, whatdir, filemask="*", parentmenu="", folders=1){
  menucount := 0
  loop, %whatdir%\*, 1, 0
  {
    if(file_isfolder(A_LoopFileFullPath)){
        if(folders){
          menucount := menu_fromfiles(A_LoopFileFullPath, a_loopfilename, whatsub, A_LoopFileFullPath, filemask, submenuname, folders)
        }
    }else{
      loop, %A_LoopFileDir%\%filemask%, 0, 0
      {
        menu, %submenuname%, add, %a_loopfilename%, %whatsub%
        menucount++
      }
    }
  }
  if(parentmenu && menucount){
    menu, %parentmenu%, add, %menutitle%, :%submenuname%
    menu, %parentmenu%, Icon, %menutitle%, %A_AhkPath%, 2
    return menucount
  }
}

;fetches the correct path from the menu
menu_itempath(whatmenu, whatdir){
    if(a_thismenu = whatmenu){
    endpath = %whatdir%\%a_thismenuitem%
        return endpath
    }else{
        endpath = %a_thismenu%\%a_thismenuitem%
        return endpath
    }
}

;returns true if the item is a folder, false if is a file
file_isfolder(whatfile){
    lastchar := substr(whatfile, 0, 1) ;fetch the last character from the string
    if(lastchar != "\")
        whatfile := whatfile . "\"
    if(fileexist(whatfile))
        return true
}

;=========================================
;======== dll hack: hide checkbox ========
;=========================================
;https://autohotkey.com/boards/viewtopic.php?t=2997

Menu_GetMenuByName(MenuName) {
   Static HMENU := 0
   If !(HMENU) {
      Menu, %A_ThisFunc%Menu, Add
      Menu, %A_ThisFunc%Menu, DeleteAll
      Gui,  %A_ThisFunc%GUI:+HwndHGUI
      Gui,  %A_ThisFunc%GUI:Menu, %A_ThisFunc%Menu
      HMENU := DllCall("User32.dll\GetMenu", "Ptr", HGUI, "UPtr")
      Gui,  %A_ThisFunc%GUI:Menu
      Gui,  %A_ThisFunc%GUI:Destroy
   }
   If !(HMENU)
      Return 0
   Menu, %A_ThisFunc%Menu, Add, :%MenuName%
   HSUBM := DllCall("User32.dll\GetSubMenu", "Ptr", HMENU, "Int", 0, "UPtr")
   Menu, %A_ThisFunc%Menu, Delete, :%MenuName%
   Return HSUBM
}

Menu_RemoveCheckMarks(HMENU, ApplyToSubMenus := True) {
   ; http://msdn.microsoft.com/en-us/library/ff468864(v=vs.85).aspx
   Static MIsize := (4 * 4) + (A_PtrSize * 3)
   VarSetCapacity(MI, MIsize, 0)
   NumPut(MIsize, MI, 0, "UInt")
   NumPut(0x00000010, MI, 4, "UInt") ; MIM_STYLE = 0x00000010
   DllCall("User32.dll\GetMenuInfo", "Ptr", HMENU, "Ptr", &MI, "UInt")

   If (ApplyToSubMenus)
      NumPut(0x80000010, MI, 4, "UInt") ; MIM_APPLYTOSUBMENUS = 0x80000000| MIM_STYLE : 0x00000010
   NumPut(NumGet(MI, 8, "UINT") | 0x80000000, MI, 8, "UInt") ; MNS_NOCHECK = 0x80000000
   DllCall("User32.dll\SetMenuInfo", "Ptr", HMENU, "Ptr", &MI, "UInt")
   Return True
}
