;=========================================
;========= progressAudio.ahk =============
;=========================================
;displays a progressbar at mouse location to adjust volume level
;two modes are available: click to adjust vs. just hover to adjust
;isClickNeeded is used to switch between these modes
;optionally: show tooltip with current value at mouse position

shortCut=F6
barW := 40
barH := 220
colBar := "Black"
colBG := "White"
rangeMin := 0
rangeMax := 100
global hasTooltip := false
global isClickNeeded := false

;=========================================
;============= OnMessage =================
;=========================================
;the OnMessage code must be on top of this file
if(isClickNeeded){
  OnMessage(0x201,"XClickableProgress")
}else{
  OnMessage(0x200,"XHoverableProgress")
}

XClickableProgress() {
 CoordMode,Mouse,Relative
 MouseGetPos,,,,ClassNN
 IfNotInString,ClassNN,msctls_progress32,return
 MouseGetPos,,,,CtrlHwnd,2
 ControlGet,Style,Style,,,ahk_id%CtrlHwnd%
 ControlGetPos,X,Y,W,H,,ahk_id%CtrlHwnd%
 VarSetCapacity(R,8)
 SendMessage,0x0407,,&R,,ahk_id%CtrlHwnd%
 R1:=NumGet(R,0,"Int"),R2:=NumGet(R,4,"Int")

 while (GetKeyState("LButton"))
 {
  MouseGetPos,XM,YM
  V := (V:=(Style&0x4 ? 1-(YM-Y)/H : (XM-X)/W))>=1 ? R2 : V<=0 ? R1 : Round(V*(R2-R1)+R1)
  if(hasTooltip){
    Tooltip % V
  }
  GuiControl,% A_Gui ":",% CtrlHwnd,% V
  ;set sound level:
  SoundSet, V
 }
}

XHoverableProgress() {
 CoordMode,Mouse,Relative
 MouseGetPos,,,,ClassNN
 IfNotInString,ClassNN,msctls_progress32,return
 MouseGetPos,,,,CtrlHwnd,2
 ControlGet,Style,Style,,,ahk_id%CtrlHwnd%
 ControlGetPos,X,Y,W,H,,ahk_id%CtrlHwnd%
 VarSetCapacity(R,8)
 SendMessage,0x0407,,&R,,ahk_id%CtrlHwnd%
 R1:=NumGet(R,0,"Int"),R2:=NumGet(R,4,"Int")

 MouseGetPos,XM,YM
 V := (V:=(Style&0x4 ? 1-(YM-Y)/H : (XM-X)/W))>=1 ? R2 : V<=0 ? R1 : Round(V*(R2-R1)+R1)
 if(hasTooltip){
   Tooltip % V
 }
 GuiControl,% A_Gui ":",% CtrlHwnd,% V

 ;set sound level:
 SoundSet, V
}

;=========================================
;============== init gui =================
;=========================================
SoundGet, vol
Gui, -Caption +AlwaysOnTop +ToolWindow -Border
Gui, Add, Progress, x0 y0 w%barW% h%barH% c%colBG% background%colBar% +0x4 range%rangeMin%-%rangeMax%,%vol%
Gui, Color, 000111

;=========================================
;============== show Gui =================
;=========================================
Hotkey,%shortCut%,show
return

show:
{
  ;get mouse Pos & current volume level
  CoordMode, Mouse, Screen
  MouseGetPos mX, mY
  SoundGet, vol

  ;calculate positioning
  mY := mY-barH+barH/rangeMax*vol-rangeMin
  mX := mX-barW/2

  ;show the gui
  Gui, Show, w%barW% h%barH% x%mX% y%mY%, Test
  WinSet, Transcolor, 000111, Test

  ;when F1 is released
  KeyWait, %shortCut%
  {
    Gui, Hide
    ToolTip
  }
}
return
