;=========================================
;========== windowSnapper.ahk ============
;=========================================
;addon that displays small rectangles to snap windows into pre-set positions
#SingleInstance force
#NoEnv

;set coordinate mode, get screenWidth & screenHeight
CoordMode, Mouse, Screen
SysGet, sW, 78
SysGet, sH, 79
global mode := 1
global numOfRects := 0

;=========================================
;=========== custom settings =============
;=========================================
shortCut=F4
global colMain := "Black"
global colHover := "White"

;PRESET: DOUBLE
;__________________
;|  1      |[__2__]  h1
;|_________|[__3__]
;d1

;PRESET: TRIPPLE
;____________________
;|  4  ||  5 ||  6  | h1
;|____||_____||____|
;t1      t2


;gui size in relation to real screen (ie. 0.5 = half of screen size)
guiSize := 0.2

;gui padding
guiP := 3

;preset double settings
d1 := 0.65
h1 := 0.5

;preset tripple settings
t1 := 0.33
t2 := 0.33

;window padding (in pixels)
p := 15

;optional: let windows overlap each other (in pixels)
overlapX := 0
overlapY := 0

;active this if you want to always take the window under the mouse,
;rather than the active one
global isHoverMode := true

;set this depending on number of available modes (currently 2)
global numOfModes := 2

;uncomment if you prefer a gui padding in relation to real screen
;guiP := p*guiSize



;=========================================
;========= Rectangles creation ===========
;=========================================
;calculate sizes for GUI
guiW := sW*guiSize
guiH := sH*guiSize

Class Rectangle {
  ;init
  __New(x, y, w, h){
    this.x := x
    this.y := y
    this.w := w
    this.h := h
  }

  ;variables
  x:=0
  y:=0
  w:=0
  h:=0
}

global Rect1
global Rect2
global Rect3
global Rect4
global Rect5
global Rect6

;create rects REAL
global Rectangles := []
Rectangles[1] := new Rectangle(p, p, d1*sW-2*p+overlapX, sH-2*p)
Rectangles[2] := new Rectangle(d1*sW-overlapX, p, (1-d1)*sW-p+overlapX, h1*sH-p+overlapY)
Rectangles[3] := new Rectangle(d1*sW-overlapX, h1*sH+p-overlapY, (1-d1)*sW-p+overlapX, (1-h1)*sH-2*p+overlapY)
Rectangles[4] := new Rectangle(p, p, t1*sW-2*p+overlapX, sH-2*p)
Rectangles[5] := new Rectangle(t1*sW-overlapX/2, p, t2*sW-p+overlapX/2, sH-2*p)
Rectangles[6] := new Rectangle((t1+t2)*sW-overlapX, p, (1-(t1+t2))*sW-p+overlapX, sH-2*p)



p := guiP
sW := guiW
sH := guiH
overlapX := 0
overlapY := 0


;create rects GUI
global GuiRectangles := []
GuiRectangles[1] := new Rectangle(p, p, d1*sW-2*p+overlapX, sH-2*p)
GuiRectangles[2] := new Rectangle(d1*sW-overlapX, p, (1-d1)*sW-p+overlapX, h1*sH-p+overlapY)
GuiRectangles[3] := new Rectangle(d1*sW-overlapX, h1*sH+p-overlapY, (1-d1)*sW-p+overlapX, (1-h1)*sH-2*p+overlapY)
GuiRectangles[4] := new Rectangle(p, p, t1*sW-2*p+overlapX, sH-2*p)
GuiRectangles[5] := new Rectangle(t1*sW-overlapX/2, p, t2*sW-p+overlapX/2, sH-2*p)
GuiRectangles[6] := new Rectangle((t1+t2)*sW-overlapX, p, (1-(t1+t2))*sW-p+overlapX, sH-2*p)






;set warning, if not all rects have been set correctly
if(Rectangles.MaxIndex() != GuiRectangles.MaxIndex()){
  msgbox % "Error: not all Rectangles[] or GuiRectangles[] have been set!"
  return
}

;set correct number of all rects
numOfRects := Rectangles.MaxIndex()



;=========================================
;============ Program start ==============
;=========================================
;--------- init the gui ---------
initGuiSnap()

;--------- bind those keys ---------
Hotkey,%shortCut%,showGuiSnap
return

showGuiSnap:
{
  ;--------- update mouse Position ---------
  ;get screen W und H (if resolution was changed etc.)
  SysGet, sW, 78
  SysGet, sH, 79
  ;get mouseCoords & current window under mouse
  MouseGetPos mX, mY, winId

  ;L
  if(mX < guiW/2){
    mX := guiW/2
  }
  ;R
  if(mX > sW-guiW/2){
    mX := sW-guiW/2
  }
  ;T
  if(mY < guiH/2){
    mY := guiH/2
  }
  ;B
  if(mY > sH-guiH/2){
    mY := sH-guiH/2
  }
  uiX := mX-guiW/2
  uiY := mY-guiH/2

  Gui, guiSnap%mode%:Show, X%uiX% Y%uiY% NoActivate

  ;--------- while shortCut down ---------
  while GetKeyState(shortCut, "P"){
    global mode
    global numOfRects

    ;RButton changes modes
    if(GetKeyState("RButton","P")){
      Send {RButton Up}
      changeMode(uiX, uiY)
    }

    ;--------- toggle fullscreen mode with L ---------
    if(GetKeyState("LButton","P")){
      ;if maximized, restore. otherwise maximize!
      WinGet, WinState, MinMax, ahk_id %winId%
      if(%WinState% != 0){
        WinRestore, ahk_id %winId%
      }else{
        Winmaximize, ahk_id %winId%
      }

      Gui, guiSnap%mode%:Hide
    }

    ;HOVER changes color
    mouseGetPos mX, mY
    MouseGetPos, , , HWIN, HCTRL, 2
    CurrControl := ""
    GuiControlGet, CurrControl, %HWIN%:Name, %HCTRL%

    Loop, %numOfRects%
    {
      currRect := "Rect" . A_Index
      if(CurrControl == currRect){
        GuiControl, guiSnap%mode%:, %currRect%, % 100
      }else{
        GuiControl, guiSnap%mode%:, %currRect%, % 0
      }
    }
  }
  ;end while shortCut DOWN


  ;--------- shortCut up ---------
  KeyWait, %shortCut%
  if(CurrControl!=""){
    ;tooltip %CurrControl%
    adjustWindow(winId, CurrControl)
  }
  Gui, guiSnap%mode%:Hide
}
return


changeMode(uiX, uiY)
{
  Gui, guiSnap%mode%:Hide

  if(mode==""){
    mode := 1
  }else{
    mode++
    if(mode>numOfModes){
      mode = 1
    }
  }

  Gui, guiSnap%mode%:Show, X%uiX% Y%uiY% NoActivate
}
return


initGuiSnap()
{
  ;--------- importing variables ---------


  Loop, %numOfRects%
  {
    ;dynamically create variable name for rect, ie. "Rect1"
    currRect := "Rect" . A_Index

    currRectX := GuiRectangles[A_Index].x
    currRectY := GuiRectangles[A_Index].y
    currRectW := GuiRectangles[A_Index].w
    currRectH := GuiRectangles[A_Index].h

    ;choose guiName
    if(A_Index == 1){
      ;DOUBLE
      Gui, guiSnap1:Color, 00FFFF
      Gui, guiSnap1:+LastFound +AlwaysOnTop +ToolWindow -Caption
      WinSet, TransColor, 00FFFF

      guiName = guiSnap1
    }else if(A_Index == 4){
      ;TRIPPLE
      Gui, guiSnap2:Color, 00FFFF
      Gui, guiSnap2:+LastFound +AlwaysOnTop +ToolWindow -Caption
      WinSet, TransColor, 00FFFF

      guiName = guiSnap2
    }

    ;add rectangles to the right gui
    Gui, %guiName%:Add, Progress, v%currRect% x%currRectX% y%currRectY% w%currRectW% h%currRectH% c%colHover% background%colMain%
  }
}
return


adjustWindow(winId, rect)
{
  ;use this to check if hoverMode or activeWindowMode
  Loop, %numOfRects%
  {
    if(rect=="Rect" . A_Index){
      if(!isHoverMode){
        WinGet, winId,, A
      }
      /*
      BORDERS
      unfortunately all windows have an additional hidden ~8px border,
      therefore -8/+8/+15 pixel modifications have to be added
      use padding = 0 to test the difference
      */


      WinMove, ahk_id %winId%,, Rectangles[A_Index].x-8, Rectangles[A_Index].y, Rectangles[A_Index].w+15, Rectangles[A_Index].h+8
    }
  }

  WinActivate, ahk_id %winId%
}
return
